/*
** File: identify.c  
** Project: ADB3 core driver
** Purpose: Identify device given Hardware ID, and set the methods structure of
**          the device context appropriately for that Hardware ID.
**
** (C) Copyright Alpha Data 2009-2010
*/

#include <df.h>

#include "device.h"
#include "device_defs.h"
#include "model_boot.h"

static boolean_t
identifyPci9656(
	Adb3CoreDeviceContext* pDevCtx,
	DfHardwareId* pId)
{
	dfDebugPrint(1, ("identifyPci9656: entered\n"));

	switch (pId->variant.pci.subDevice) {
	case PCI_SUBDEVICE_PCI9656_ADMXRC2:
		dfDebugPrint(0, ("identifyPci9656: identified ADM-XRC-II\n"));
		pDevCtx->methods.pModelBootstrap = adb3CoreBootstrapAdmxrc2;
		return TRUE;

  default:
		dfDebugPrint(0, ("*** identifyPci9656: unknown PCI9656 device\n"));
		return FALSE;
	}
}

static boolean_t
identifyPlxTech(
	Adb3CoreDeviceContext* pDevCtx,
	DfHardwareId* pId)
{
	dfDebugPrint(1, ("identifyAlphaData: entered\n"));

	switch (pId->variant.pci.device) {
	case PCI_DEVICE_PLXTECH_PCI9656:
		dfDebugPrint(1, ("identifyAlphaData: identified PCI9656\n"));
    return identifyPci9656(pDevCtx, pId);

  default:
		break;
	}

	dfDebugPrint(0, ("***  identifyAlphaData: failed to recognize device\n"));

	return FALSE;
}

static boolean_t
identifyAdb3(
	Adb3CoreDeviceContext* pDevCtx,
	DfHardwareId* pId)
{
	dfDebugPrint(1, ("identifyAdb3: entered\n"));

	switch (pId->variant.pci.subDevice) {
	case PCI_SUBDEVICE_ADB3_ADPEXRC5T:
		dfDebugPrint(0, ("identifyAdb3: identified ADPe-XRC-5T\n"));
		pDevCtx->methods.pModelBootstrap = adb3CoreBootstrapAdpexrc5t;
		return TRUE;

	case PCI_SUBDEVICE_ADB3_ADMXRC6TL_LX50T:
		dfDebugPrint(0, ("identifyAdb3: identified ADM-XRC-6TL (5VLX50T bridge)\n"));
		pDevCtx->methods.pModelBootstrap = adb3CoreBootstrapAdmxrc6tl;
		return TRUE;

  case PCI_SUBDEVICE_ADB3_ADMXRC6TL_FX50T:
		dfDebugPrint(0, ("identifyAdb3: identified ADM-XRC-6TL (5VFX70T bridge)\n"));
		pDevCtx->methods.pModelBootstrap = adb3CoreBootstrapAdmxrc6tl;
		return TRUE;

  case PCI_SUBDEVICE_ADB3_ADMXRC6T1:
		dfDebugPrint(0, ("identifyAdb3: identified ADM-XRC-6T1\n"));
		pDevCtx->methods.pModelBootstrap = adb3CoreBootstrapAdmxrc6t1;
		return TRUE;

  case PCI_SUBDEVICE_ADB3_ADMXRC6T1_BERT:
		dfDebugPrint(0, ("identifyAdb3: identified ADM-XRC-6T1 (BERT configuration)\n"));
		pDevCtx->methods.pModelBootstrap = adb3CoreBootstrapAdmxrc6t1;
		return TRUE;

  default:
		dfDebugPrint(0, ("identifyAdb3: identified generic ADB3 device\n"));
		pDevCtx->methods.pModelBootstrap = adb3CoreBootstrapGenericAdb3;
		return TRUE;
	}
}

static boolean_t
identifyAlphaData(
	Adb3CoreDeviceContext* pDevCtx,
	DfHardwareId* pId)
{
	dfDebugPrint(1, ("identifyAlphaData: entered\n"));

	switch (pId->variant.pci.device) {
	case PCI_DEVICE_ADATA_ADB3:
		dfDebugPrint(1, ("identifyAlphaData: identified ADB3\n"));
    return identifyAdb3(pDevCtx, pId);

	default:
		break;
	}

	dfDebugPrint(0, ("***  identifyAlphaData: failed to recognize device\n"));

	return FALSE;
}

boolean_t
identifyDevice(
	Adb3CoreDeviceContext* pDevCtx,
	DfHardwareId* pId)
{
	dfDebugPrint(1, ("identifyDevice: entered\n"));

	if (pId->busType == DfBusPci) {
    switch (pId->variant.pci.vendor) {
    case PCI_VENDOR_ADATA:
    	dfDebugPrint(1, ("identifyDevice: vendor is ALPHA DATA\n"));
			return identifyAlphaData(pDevCtx, pId);

    case PCI_VENDOR_PLXTECH:
    	dfDebugPrint(1, ("identifyDevice: vendor is PLX Tech.\n"));
			return identifyPlxTech(pDevCtx, pId);

    default:
    	/* Not recognized */
      return FALSE;
    }
	}

  /* Bus type not supported */
  dfDebugPrint(0, ("*** identifyDevice: unrecognised bus type %lu (0x%lx)\n", (unsigned long)pId->busType, (unsigned long)pId->busType));
  return FALSE;
}
