/*
** File: dfcommon.h  
** Project: Driver Framework
** Purpose: Platform-independent definitions
**
** (C) Copyright Alpha Data 2009-2010
*/

#if !defined(ADATA_DF_DFCOMMON_H)
#define ADATA_DF_DFCOMMON_H

#define DF_OFFSET_OF(STRUCT, MEMBER) \
  ((size_t)&((STRUCT*)0)->MEMBER)

#define DF_CONTAINER_OF(VARIABLE, STRUCT, MEMBER) \
  ((STRUCT*)((uint8_t*)(VARIABLE) - DF_OFFSET_OF(STRUCT, MEMBER)))

/* Aligns a pointer to specified alignment */
#define DF_ALIGN_POINTER(x, align) \
	(void*)(((uintptr_t)(x) + (align - 1U)) & ~(uintptr_t)(align - 1U))

/* Returns offset by which a pointer is misaligned to specified alignment */
#define DF_POINTER_ALIGNMENT(x, align) \
	(((uintptr_t)(x)) & (uintptr_t)((align) - 1U))

#define DF_ARRAY_LENGTH(x) (sizeof(x) / sizeof((x)[0]))

/*
** Bit field conventions to be used in drivers:
**
** xxx_L_yyy_zzz is a field length, in bits
** xxx_S_yyy_zzz is a field shift
** xxx_M_yyy_zzz is a field mask, shifted by field shift
** xxx_V_yyy_zzz is a field value, shifted by field shift
**
** where xxx identifies a set of registers
**       yyy identifies a register
**       zzz identifies a field in a register
**
** DF_MASKxU creates an unshifted mask from field length
** DF_MASKxS creates a shifted mask from field length and field shift
** DF_VALxS  creates a shifted value from field value, field length and field shift
*/

#define DF_MASK8U(length)  (( uint8_t)((1U << (length)) - 1))
#define DF_MASK16U(length) ((uint16_t)((1U << (length)) - 1))
#define DF_MASK32U(length) ((uint32_t)((1U << (length)) - 1))
#define DF_MASK64U(length) ((uint64_t)((1U << (length)) - 1))

#define DF_MASK8S(shift, length)  (( uint8_t)( DF_MASK8U(length) << (shift)))
#define DF_MASK16S(shift, length) ((uint16_t)(DF_MASK16U(length) << (shift)))
#define DF_MASK32S(shift, length) ((uint32_t)(DF_MASK32U(length) << (shift)))
#define DF_MASK64S(shift, length) ((uint64_t)(DF_MASK64U(length) << (shift)))

#define DF_VAL8S(val, shift, length)  (( uint8_t)(((val) &  DF_MASK8U(length)) << shift))
#define DF_VAL16S(val, shift, length) ((uint16_t)(((val) & DF_MASK16U(length)) << shift))
#define DF_VAL32S(val, shift, length) ((uint32_t)(((val) & DF_MASK32U(length)) << shift))
#define DF_VAL64S(val, shift, length) ((uint64_t)(((val) & DF_MASK64U(length)) << shift))

#endif
